#!/bin/bash

##
## Name:
##      qacct.sh
##
## Version:
##      $Revision: 1.2 $
##
## Purpose:
##      Create a .qmail file with a random extension. Possible uses
##      include creating unique addresses for online purchases (find out
##      who's selling your email address for spam and profit!) or making
##      "disposable" addresses that can be deleted whenever the spam
##      volume for a particular address gets too high.
##
## Usage:
##      qacct.sh [-u | -h]
##      qacct.sh [-l <length>] [-d <directory>]
##
## Options:
##      -d = directory for .qmail files (default: $HOME)
##      -h = show documentation
##      -l = length of extension (default: 9 characters)
##      -u = show usage
##
## Errorlevels:
##      0 = Success
##      1 = Failure
##      2 = Other
##
## Notes:
##      If you want to ensure that spammers can't abuse your "real"
##      address just by leaving off the random extension, just add the
##      following line to your $HOME/.qmail file:
##
##          |bouncesaying "Sorry, no mailbox here by that name. (#5.1.1)"
##
##      This way, only your disposable addresses will allow delivery,
##      since any mail delivered to the un-extended address will bounce
##      as if the account didn't exist.
##
##      And don't forget, if bouncesaying is not in your path, you may
##      need to include the full path to the binary. This will usually
##      be /var/qmail/bin/bouncesaying.
##
## Copyright:
##      Copyright (c) 2001 by Todd A. Jacobs <bash_junkie@codegnome.org>
##      All Rights Reserved
##
## License:
##      Released under the GNU General Public License (GPL)
##      http://www.gnu.org/copyleft/gpl.html
##
##      This program is free software; you can redistribute it and/or
##      modify it under the terms of the GNU General Public License
##      as published by the Free Software Foundation; either version 2
##      of the License, or (at your option) any later version.
##
##      This program is distributed in the hope that it will be useful,
##      but WITHOUT ANY WARRANTY; without even the implied warranty of
##      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##      GNU General Public License for more details.
##


######################################################################
# Constants
######################################################################

# Zero-based array of characters from which to draw randomly. Note that
# qmail lowercases all inbound addresses, and will fail to match
# uppercase dot-qmail extensions, so we must ensure that all letters in
# the extension are lowercase.
Character_Pool=(a b c d e f g h i j k l m n o p q r s t u v w x y z \
    0 1 2 3 4 5 6 7 8 9)

# Default number of characters in dot-qmail extension.
Extension_Length=9

# The directory into which the new .qmail files will be placed.
Working_Directory=$HOME


######################################################################
# Functions
######################################################################

# Show brief help on usage.
function Show_Usage {
        echo -n "Usage: "
        egrep -A2 "^## Usage" $0 | tail -n1 | sed -e 's/^##[[:space:]]*//'
        exit 2
}

# Select random characters one at a time until the value passed as
# positional parameter 1 has been reached.
function Get_Random_String {
    Num_Characters=0
    while [ $Num_Characters -lt $Extension_Length ]; do
        let Random_Number="$RANDOM % 36"
        echo -n ${Character_Pool[$Random_Number]}
        let Num_Characters+=1
    done
}


######################################################################
# Process Command-Line Options
######################################################################
while getopts ":hud:l:" opt; do
    case $opt in
        d)
            # Override default directory after doing a little error
            # checking to make sure it's both a directory and
            # writable by the current user. This option is
            # especially useful for creating aliases in the ~alias
            # directory.
            if [ -d $OPTARG -a -w $OPTARG ]; then
                Working_Directory=$OPTARG
            else
                echo "$OPTARG is not a writable directory."
                exit 1
            fi
            ;;
        l)
            # Override the default number of characters in the
            # .qmail extension.
            Extension_Length=$OPTARG
            ;;
        h)
            # If passed "-h" for help, use egrep to show the help
            # comments designated by an inital double-octothorpe and
            # exit with errorlevel 2.
            egrep '^##[^#]' $0 | sed 's/##//'
            exit 2
            ;;
        \? | u)
            Show_Usage
            ;;
    esac # End "case $opt"
done # End "while getopts"


######################################################################
# Main
######################################################################

# Loop until a unique extension has been created.
while true; do
    New_Extension=$(Get_Random_String $Extension_Length)

    # If file already exists, don't clobber it. Instead, restart the
    # loop and try again.
    [ -e $Working_Directory/.qmail-$New_Extension ] && continue

    # Do a little error checking on the file creation.
    if ! touch $Working_Directory/.qmail-$New_Extension; then
        echo "Failed to create $(id -ng)-$New_Extension@${HOSTNAME#*.}"
        exit 1
    fi

    # Let the user know the script succeeded, and output the email
    # address for cut-and-paste purposes.
    echo "Created file:     $Working_Directory/.qmail-$New_Extension"
    echo "Created address:  $(id -ng)-$New_Extension@${HOSTNAME#*.}"
    break
done
